<?php

namespace ContentEgg\application;

/**
 * ProductSearchWidget class file
 *
 * @author keywordrush.com <support@keywordrush.com>
 * @link http://www.keywordrush.com/
 * @copyright Copyright &copy; 2017 keywordrush.com
 */
class ProductSearchWidget extends \WP_Widget {

    public static function initAction()
    {
        \add_action('widgets_init', function() {
            \register_widget(__CLASS__);
        });
    }

    /**
     * Register widget with WordPress.
     */
    public function __construct()
    {
        parent::__construct(
                'cegg_product_search', \esc_html__('CE:Product Search', 'content-egg'), array('description' => \esc_html__('A search form for affiliate products.', 'content-egg'), 'classname' => 'widget widget_search')
        );
    }

    /**
     * Front-end display of widget.
     *
     */
    public function widget($args, $instance)
    {
        $title = \apply_filters('widget_title', empty($instance['title']) ? '' : $instance['title'], $instance, $this->id_base);

        echo $args['before_widget'];
        if ($title)
        {
            echo $args['before_title'] . $title . $args['after_title'];
        }

        // Use current theme search form if it exists
        echo self::getSearchForm();

        echo $args['after_widget'];
    }

    /**
     * Back-end widget form.
     *
     */
    public function form($instance)
    {
        $title = !empty($instance['title']) ? $instance['title'] : '';
        ?>
        <p>
            <label for="<?php echo \esc_attr($this->get_field_id('title')); ?>"><?php \esc_attr_e('Title:', 'content-egg'); ?></label> 
            <input class="widefat" id="<?php echo \esc_attr($this->get_field_id('title')); ?>" name="<?php echo \esc_attr($this->get_field_name('title')); ?>" type="text" value="<?php echo \esc_attr($title); ?>">
        </p>
        <?php
    }

    /**
     * Sanitize widget form values as they are saved.
     */
    public function update($new_instance, $old_instance)
    {
        $instance = array();
        $instance['title'] = (!empty($new_instance['title']) ) ? sanitize_text_field($new_instance['title']) : '';

        return $instance;
    }

    public static function getSearchForm()
    {
        $search_form_template = \locate_template('ce-product-searchform.php');
        if ($search_form_template)
        {
            ob_start();
            require( $search_form_template );
            $form = ob_get_clean();
        } else
        {
            // standart wp search from
            $form = \get_search_form(false);

            if (\get_option('permalink_structure'))
                $form = preg_replace('/action=["\'].+?["\']/', 'action="' . self::getSearchFormUri() . '"', $form);
            else
                $form = preg_replace('/<\/form>/', '<input type="hidden" name="pagename" value="' . \esc_attr(ProductSearch::PAGE_SLUG) . '"></form>', $form);

            $form = preg_replace('/placeholder=".+?"/', 'placeholder="' . esc_attr(__('Product Search...', 'content-egg-tpl')) . '"', $form);
        }
        return $form;
    }

    public static function getSearchFormUri()
    {
        if (\get_option('permalink_structure'))
            return \esc_url(\home_url(ProductSearch::PAGE_SLUG));
        else
            return \esc_url(\home_url('/'));
    }

}
